@@ -1,4 +1,6 @@
 """TeDa FITS Viewer main window"""
+import os
+
 import PySide2
 from PySide2 import QtWidgets, QtCore
 from PySide2.QtCore import QFile, Qt, QTextStream, QSettings
@@ -28,6 +30,7 @@ from teda.models.cmaps import ColorMaps
 from teda.models.scalesModel import ScalesModel
 from teda.icons import IconFactory
 from . import console
+from .widgets.fileSystemWidget import FileSystemWidget
 
 
 class MainWindow(QMainWindow):
@@ -60,6 +63,7 @@ class MainWindow(QMainWindow):
 
         self.painterComponent = PainterComponent(self.fits_image)
         self.painterComponent.startMovingEvents(self.central_widget)
+        self.scanObject = ScanToolbar(self)
         self.createActions()
         self.createMenus()
         self.createToolBars()
@@ -144,6 +148,20 @@ class MainWindow(QMainWindow):
 
         self.statusBar().showMessage("Saved '%s'" % filename, 2000)
 
+    def save_dialog(self):
+        figure = self.central_widget.figure
+        filetypes = figure.canvas.get_supported_filetypes_grouped()
+        filterstr = ';;'.join([
+            k+' (' + ' '.join([
+                '*.'+ext for ext in v
+            ])+')' for k, v in filetypes.items()
+        ])
+        dialog = QFileDialog.getSaveFileName(self, "Save Image As...", os.path.splitext(self.filename)[0], filterstr)
+        if dialog[0] != "":
+            try:
+                self.central_widget.figure.savefig(dialog[0])
+            except ValueError:
+                print("Unsupported format")
 
     def open_fits(self, fileName):
         """Opens specified FITS file and loads it to user interface"""
@@ -227,7 +245,18 @@ class MainWindow(QMainWindow):
                           "<li>Edward Lis</li>"
                           "<li>Grzegorz Mroczkowski</li>"
                           "</ul>"
-                          "Created by <a href='https://akond.com'>Akond Lab</a> for The Araucaria Project")
+                          "Created by <a href='https://akond.com'>Akond Lab</a> for The "
+                          "<a href='https://araucaria.camk.edu.pl'>Araucaria Project</a><br/>"
+                          "Licence: MIT <br/>"
+                          "3rd party work used: "
+                          "<a href='https://material.io/resources/icons/'> Google Material Icons</a>, "
+                          "<a href='https://www.astropy.org'> AstroPy</a>, "
+                          "<a href='https://doc.qt.io/qtforpython/'> Qt5/PySide2</a>, "
+                          "<a href='https://www.scipy.org'> SciPy</a>, and other..."
+                          "<br/><br/>"
+                          "Visit the <a href='https://github.com/majkelx/teda'>project's GitHub  page</a> for help"
+                          " and the issue tracker"
+                          )
 
     def on_console_show(self):
         console.show(
@@ -249,6 +278,9 @@ class MainWindow(QMainWindow):
         # self.openAct = QAction(ico1, "&Open", self, shortcut=QKeySequence.Open, statusTip="Open FITS file", triggered=self.open)
         self.openAct = QAction(IconFactory.getIcon('note_add'),
                                "&Open", self, shortcut=QKeySequence.Open, statusTip="Open FITS file", triggered=self.open_dialog)
+        self.saveAct = QAction(IconFactory.getIcon('save'),
+                               "&Save", self, shortcut=QKeySequence.Save, statusTip="Save FITS view",
+                               triggered=self.save_dialog)
         self.quitAct = QAction("&Quit", self, shortcut="Ctrl+Q", statusTip="Quit the application", triggered=self.close)
         self.aboutAct = QAction("&About", self, statusTip="Show the application's About box", triggered=self.about)
         self.aboutQtAct = QAction("About &Qt", self, statusTip="Show the Qt library's About box", triggered=QApplication.instance().aboutQt)
@@ -271,17 +303,19 @@ class MainWindow(QMainWindow):
         self.nextHDUAct = QAction(IconFactory.getIcon('skip_next'), 'Next HDU', self,
                                   statusTip="Next HDU", triggered=self.nextHDU)
 
-        self.zoom4Act = QAction(IconFactory.getIcon(), '×4', self,
+        self.zoom4Act = QAction(IconFactory.getIcon("x4"), 'Zoom ×4', self,
                                   statusTip="Zoom ×4", triggered=self.setZoomButton4)
-        self.zoom2Act = QAction(IconFactory.getIcon(), '×2', self,
+        self.zoom2Act = QAction(IconFactory.getIcon("x2"), 'Zoom ×2', self,
                                   statusTip="Zoom ×2", triggered=self.setZoomButton2)
         self.zoomHomeAct = QAction(IconFactory.getIcon('home'), 'Home', self,
                                   statusTip="Reset zoom an position", triggered=self.setZoomButtonHome)
-        self.zoom05Act = QAction(IconFactory.getIcon(), '1/2', self,
+        self.zoom05Act = QAction(IconFactory.getIcon("1-2"), 'Zoom 1/2', self,
                                   statusTip="Zoom 1/2", triggered=self.setZoomButton05)
-        self.zoom025Act = QAction(IconFactory.getIcon(), '1/4', self,
+        self.zoom025Act = QAction(IconFactory.getIcon("1-4"), 'Zoom 1/4', self,
                                   statusTip="Zoom 1/4", triggered=self.setZoomButton025)
 
+        self.panningAct = QAction(IconFactory.getIcon('panning'), 'Panning', self,
+                                 statusTip="Panning", triggered=self.changePanningStatus)
         self.circleAct = QAction(IconFactory.getIcon('circle'), 'Add Region', self,
                                   statusTip="Add Region", triggered=self.changeAddCircleStatus)
         self.centerCircleAct = QAction(IconFactory.getIcon('add_circle_outline'), 'Radial profile', self,
@@ -292,10 +326,35 @@ class MainWindow(QMainWindow):
     def createMenus(self):
         self.fileMenu = self.menuBar().addMenu("&File")
         self.fileMenu.addAction(self.openAct)
-
+        self.fileMenu.addAction(self.scanObject.scanAct)
+        self.fileMenu.addAction(self.scanObject.stopAct)
+        self.fileMenu.addAction(self.scanObject.pauseAct)
+        self.fileMenu.addAction(self.scanObject.resumeAct)
+        self.fileMenu.addAction(self.scanObject.autopauseAct)
+        self.fileMenu.addAction(self.scanObject.disabledautopauseAct)
+        self.fileMenu.addAction(self.saveAct)
         self.fileMenu.addSeparator()
         self.fileMenu.addAction(self.quitAct)
 
+        self.editMenu = self.menuBar().addMenu("&Edit")
+        self.editMenu.addAction(self.panningAct)
+        self.editMenu.addAction(self.circleAct)
+        self.editMenu.addAction(self.centerCircleAct)
+        self.editMenu.addSeparator()
+        self.editMenu.addAction(self.deleteAct)
+
+        self.hduMenu = self.menuBar().addMenu("HDU")
+        self.hduMenu.addAction(self.prevHDUAct)
+        self.hduMenu.addAction(self.nextHDUAct)
+        self.hduMenu.addSeparator()
+
+        self.zoomMenu = self.menuBar().addMenu("Zoom")
+        self.zoomMenu.addAction(self.zoom4Act)
+        self.zoomMenu.addAction(self.zoom2Act)
+        self.zoomMenu.addAction(self.zoomHomeAct)
+        self.zoomMenu.addAction(self.zoom05Act)
+        self.zoomMenu.addAction(self.zoom025Act)
+
         self.WcsMenu = self.menuBar().addMenu("W&CS")
         self.WcsMenu.addAction(self.wcsSexAct)
         self.WcsMenu.addSeparator()
@@ -314,17 +373,19 @@ class MainWindow(QMainWindow):
     def createToolBars(self):
         self.fileToolBar = self.addToolBar("File Toolbar")
         self.fileToolBar.addAction(self.openAct)
+        self.fileToolBar.addAction(self.saveAct)
 
         self.hduToolBar = self.addToolBar("HDU Toolbar")
         self.hduToolBar.addAction(self.prevHDUAct)
         self.hduToolBar.addAction(self.nextHDUAct)
 
         self.scanToolBar = self.addToolBar("Scan Toolbar")
-        self.scanObject = ScanToolbar(self)
         self.scanToolBar.addAction(self.scanObject.scanAct)
         self.scanToolBar.addAction(self.scanObject.stopAct)
         self.scanToolBar.addAction(self.scanObject.pauseAct)
         self.scanToolBar.addAction(self.scanObject.resumeAct)
+        self.scanToolBar.addAction(self.scanObject.autopauseAct)
+        self.scanToolBar.addAction(self.scanObject.disabledautopauseAct)
         self.scanToolBar.hide()
 
         # self.infoToolBar = self.addToolBar("Info Toolbar")
@@ -344,6 +405,9 @@ class MainWindow(QMainWindow):
         self.zoomToolBar.addAction(self.zoom025Act)
 
         self.mouseActionToolBar = self.addToolBar("Mouse Task Toolbar")
+        self.panningAct.setCheckable(True)
+        self.panningAct.setChecked(True)
+        self.mouseActionToolBar.addAction(self.panningAct)
         self.circleAct.setCheckable(True)
         self.mouseActionToolBar.addAction(self.circleAct)
         self.centerCircleAct.setCheckable(True)
@@ -386,6 +450,16 @@ class MainWindow(QMainWindow):
             self.fits_image.setZoom(zoom, reset)
             self.full_view_widget.updateMiniatureShape(self.fits_image.viewX, self.fits_image.viewY, self.fits_image.viewW, self.fits_image.viewH)
 
+    def changePanningStatus(self):
+        if self.panningAct.isChecked():
+            self.toogleOffRegionButtons()
+            self.panningAct.toggle()
+            self.painterComponent.stopPainting(self.central_widget)
+            self.painterComponent.startMovingEvents(self.central_widget)
+        else:
+            self.painterComponent.stopPainting(self.central_widget)
+            self.painterComponent.stopMovingEvents(self.central_widget)
+
     def changeAddCircleStatus(self):
         if self.circleAct.isChecked():
             self.toogleOffRegionButtons()
@@ -394,6 +468,7 @@ class MainWindow(QMainWindow):
         else:
             self.painterComponent.stopPainting(self.central_widget)
             self.painterComponent.startMovingEvents(self.central_widget)
+            self.panningAct.toggle()
 
     def changeAddCenterCircleStatus(self):
         if self.centerCircleAct.isChecked():
@@ -403,11 +478,14 @@ class MainWindow(QMainWindow):
         else:
             self.painterComponent.stopPainting(self.central_widget)
             self.painterComponent.startMovingEvents(self.central_widget)
+            self.panningAct.toggle()
 
     def deleteSelected(self):
         self.painterComponent.deleteSelectedShapes(self.central_widget.figure.axes[0])
 
     def toogleOffRegionButtons(self):
+        if self.panningAct.isChecked():
+            self.panningAct.toggle()
         if self.circleAct.isChecked():
             self.circleAct.toggle()
         if self.centerCircleAct.isChecked():
@@ -495,6 +573,15 @@ class MainWindow(QMainWindow):
         self.addDockWidget(Qt.TopDockWidgetArea, dock)
         self.viewMenu.addAction(dock.toggleViewAction())
 
+        # fileSelector
+        dock = QDockWidget("Directory view", self)
+        dock.setObjectName("DIRECTORY_VIEW")
+        dock.setAllowedAreas(Qt.LeftDockWidgetArea)
+        self.file_widget = FileSystemWidget(self)
+        dock.setWidget(self.file_widget)
+        self.addDockWidget(Qt.LeftDockWidgetArea, dock)
+        self.viewMenu.addAction(dock.toggleViewAction())
+
         self.viewMenu.addSeparator()
 
 
@@ -553,6 +640,11 @@ class MainWindow(QMainWindow):
             self.cursor_coords.set_img_y(change.new)
         if display != '':
             self.zoom_view_widget.setXYofZoom(self.fits_image, self.current_x_coord, self.current_y_coord, self.fits_image.zoom)
+            if not self.hasFocus():
+                self.setFocus()
+            if self.scanObject.activeScan and self.scanObject.enableAutopause:#reser autopause
+                if not self.scanObject.obserwableValue.autopauseFlag:
+                    self.scanObject.obserwableValue.autopauseFlag = True
 
     def onMouseZoomOnImage(self, change):
         changed = False
@@ -585,6 +677,7 @@ class MainWindow(QMainWindow):
             self.restoreState(settings.value("windowState"))
 
         self.headerWidget.readSettings(settings)
+        self.file_widget.readSettings(settings)
 
     def writeWindowSettings(self):
         if self.tedaCommandLine.ignoreSettings:
@@ -599,6 +692,7 @@ class MainWindow(QMainWindow):
         settings.setValue('windowState',self.saveState())
 
         self.headerWidget.writeSettings(settings)
+        self.file_widget.writeSettings(settings)
 
     # def updateFitsInWidgets(self):
     #     # print("updateFitsInWidgets")
